@@ -16,18 +16,24 @@ package Image::ExifTool::GPS;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '1.00';
+$VERSION = '1.04';
 
 %Image::ExifTool::GPS::Main = (
+    WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
+    CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
+    WRITABLE => 1,
     GROUPS => { 2 => 'Location' },
     0x0000 => {
         Name => 'GPSVersionID',
-        ValueConv => '$val =~ s/\0$//; $val',
+        Writable => 'int8u',
+        Count => 4,
         PrintConv => '$val =~ tr/ /./; $val',
+        PrintConvInv => '$val =~ tr/./ /; $val',
     },
     0x0001 => {
         Name => 'GPSLatitudeRef',
-        ValueConv => '$val =~ s/\0$//; $val',
+        Writable => 'string',
+        Count => 2,
         PrintConv => {
             N => 'North',
             S => 'South',
@@ -35,11 +41,15 @@ $VERSION = '1.00';
     },
     0x0002 => {
         Name => 'GPSLatitude',
+        Writable => 'rational32u',
+        Count => 3,
         PrintConv => 'Image::ExifTool::GPS::DMS($val)',
+        PrintConvInv => '$_=$val;tr/-+0-9.\t/ /c;$_',
     },
     0x0003 => {
         Name => 'GPSLongitudeRef',
-        ValueConv => '$val =~ s/\0$//; $val',
+        Writable => 'string',
+        Count => 2,
         PrintConv => {
             E => 'East',
             W => 'West',
@@ -47,28 +57,40 @@ $VERSION = '1.00';
     },
     0x0004 => {
         Name => 'GPSLongitude',
+        Writable => 'rational32u',
+        Count => 3,
         PrintConv => 'Image::ExifTool::GPS::DMS($val)',
+        PrintConvInv => '$_=$val;tr/-+0-9.	/ /c;$_',
     },
     0x0005 => {
         Name => 'GPSAltitudeRef',
+        Writable => 'int8u',
         # don't need to remove "\0" here -- not a string
         PrintConv => {
-            0 => 'Sea Level',
+            0 => 'Above Sea Level',
+            1 => 'Below Sea Level',
         },
     },
     0x0006 => {
         Name => 'GPSAltitude',
+        Writable => 'rational32u',
         PrintConv => '"$val metres"',
     },
     0x0007 => {
         Name => 'GPSTimeStamp',
+        Writable => 'rational32u',
+        Count => 3,
         Groups => { 2 => 'Time' },
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
     },
-    0x0008 => 'GPSSatellites',
+    0x0008 => {
+        Name => 'GPSSatellites',
+        Writable => 'string',
+    },
     0x0009 => {
         Name => 'GPSStatus',
-        ValueConv => '$val =~ s/\0$//; $val',
+        Writable => 'string',
+        Count => 2,
         PrintConv => {
             A => 'Measurement In Progress',
             V => 'Measurement Interoperability',
@@ -76,45 +98,65 @@ $VERSION = '1.00';
     },
     0x000A => {
         Name => 'GPSMeasureMode',
-        ValueConv => '$val =~ s/\0$//; $val',
+        Writable => 'string',
+        Count => 2,
         PrintConv => {
             2 => '2-Dimensional Measurement',
             3 => '3-Dimensional Measurement',
         },
     },
-    0x000B => 'GPSDOP',
+    0x000B => {
+        Name => 'GPSDOP',
+        Writable => 'rational32u',
+    },
     0x000C => {
         Name => 'GPSSpeedRef',
-        ValueConv => '$val =~ s/\0$//; $val',
+        Writable => 'string',
+        Count => 2,
         PrintConv => {
             K => 'km/h',
             M => 'mph',
             N => 'knots',
         },
     },
-    0x000D => 'GPSSpeed',
+    0x000D => {
+        Name => 'GPSSpeed',
+        Writable => 'rational32u',
+    },
     0x000E => {
         Name => 'GPSTrackRef',
-        ValueConv => '$val =~ s/\0$//; $val',
+        Writable => 'string',
+        Count => 2,
         PrintConv => {
             M => 'Magnetic North',
             T => 'True North',
         },
     },
-    0x000F => 'GPSTrack',
+    0x000F => {
+        Name => 'GPSTrack',
+        Writable => 'rational32u',
+    },
     0x0010 => {
         Name => 'GPSImgDirectionRef',
-        ValueConv => '$val =~ s/\0$//; $val',
+        Writable => 'string',
+        Count => 2,
         PrintConv => {
             M => 'Magnetic North',
             T => 'True North',
         },
     },
-    0x0011 => 'GPSImgDirection',
-    0x0012 => 'GPSMapDatum',
+    0x0011 => {
+        Name => 'GPSImgDirection',
+        Writable => 'rational32u',
+    },
+    0x0012 => {
+        Name => 'GPSMapDatum',
+        Writable => 'string',
+    },
     0x0013 => {
         Name => 'GPSDestLatitudeRef',
-        ValueConv => '$val =~ s/\0$//; $val',
+        Writable => 'string',
+        Count => 2,
         PrintConv => {
             N => 'North',
             S => 'South',
@@ -122,11 +164,15 @@ $VERSION = '1.00';
     },
     0x0014 => {
         Name => 'GPSDestLatitude',
+        Writable => 'rational32u',
+        Count => 3,
         PrintConv => 'Image::ExifTool::GPS::DMS($val)',
+        PrintConvInv => '$_=$val;tr/-+0-9.	/ /c;$_',
     },
     0x0015 => {
         Name => 'GPSDestLongitudeRef',
-        ValueConv => '$val =~ s/\0$//; $val',
+        Writable => 'string',
+        Count => 2,
         PrintConv => {
             E => 'East',
             W => 'West',
@@ -134,36 +180,60 @@ $VERSION = '1.00';
     },
     0x0016 => {
         Name => 'GPSDestLongitude',
+        Writable => 'rational32u',
+        Count => 3,
         PrintConv => 'Image::ExifTool::GPS::DMS($val)',
+        PrintConvInv => '$_=$val;tr/-+0-9.	/ /c;$_',
     },
     0x0017 => {
         Name => 'GPSDestBearingRef',
-        ValueConv => '$val =~ s/\0$//; $val',
+        Writable => 'string',
+        Count => 2,
         PrintConv => {
             M => 'Magnetic North',
             T => 'True North',
         },
     },
-    0x0018 => 'GPSDestBearing',
+    0x0018 => {
+        Name => 'GPSDestBearing',
+        Writable => 'rational32u',
+    },
     0x0019 => {
         Name => 'GPSDestDistanceRef',
-        ValueConv => '$val =~ s/\0$//; $val',
+        Writable => 'string',
+        Count => 2,
         PrintConv => {
             K => 'Kilometers',
             M => 'Miles',
             N => 'Nautical Miles',
         },
     },
-    0x001A => 'GPSDestDistance',
-    0x001B => 'GPSProcessingMethod',
-    0x001C => 'GPSAreaInformation',
+    0x001A => {
+        Name => 'GPSDestDistance',
+        Writable => 'rational32u',
+    },
+    0x001B => {
+        Name => 'GPSProcessingMethod',
+        Writable => 'undef',
+        PrintConv => '$_=substr($val,8);s/\0.*//;$_',
+        PrintConvInv => '"ASCII\0\0\0$val\0"',
+    },
+    0x001C => {
+        Name => 'GPSAreaInformation',
+        Writable => 'undef',
+        PrintConv => '$_=substr($val,8);s/\0.*//;$_',
+        PrintConvInv => '"ASCII\0\0\0$val\0"',
+    },
     0x001D => {
         Name => 'GPSDateStamp',
+        Writable => 'string',
+        Count => 11,
         Groups => { 2 => 'Time' },
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
     },
     0x001E => {
         Name => 'GPSDifferential',
+        Writable => 'int16u',
         PrintConv => {
             0 => 'No Correction',
             1 => 'Differential Corrected',
@@ -213,7 +283,7 @@ GPS (Global Positioning System) meta information in EXIF data.
 
 =head1 AUTHOR
 
-Copyright 2003-2004, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2005, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
